import { Broker } from './broker';
import { InsightRequest, InsightResponse, InsightType } from '../store/insightSharingSlice';

type GetInsightsResponse = (data: { reports: InsightResponse[]; alerts: InsightResponse[] }, status: string) => void;
export function wsGetInsights(saveStateId: string, callback?: GetInsightsResponse) {
  const internalCallback: GetInsightsResponse = (data, status) => {
    if (callback) callback(data, status);
  };

  Broker.instance().sendMessage(
    {
      key: 'insight',
      subKey: 'getAll',
      body: { saveStateId: saveStateId }
    },
    internalCallback
  );
}

export function wsCreateInsight(insight: InsightRequest, callback?: Function) {
  Broker.instance().sendMessage(
    {
      key: 'insight',
      subKey: 'create',
      body: insight,
    },
    (data: InsightResponse, status: string) => {
      if (status === 'Bad Request') {
        console.error('Failed to create insight:', data);
        if (callback) callback(data, status);
        return;
      }

      if (!data || typeof data !== 'object') {
        console.error('Invalid response data', data)
        if (callback) callback(null, 'error');
        return;
      }

      if (!data.type || !data.id) {
        console.error('Missing fields in response', data)
        if (callback) callback(null, 'error');
        return;
      }

      if (callback) callback(data, status);
    }
  );
}

export function wsUpdateInsight(
  id: number,
  insight: InsightRequest,
  generateEmail: boolean,
  callback?: (data: any, status: string) => void
) {
  Broker.instance().sendMessage(
    {
      key: 'insight',
      subKey: 'update',
      body: { id: id, insight: insight, generateEmail: generateEmail },
    },
    (data: any, status: string) => {
      if (status === 'Bad Request') {
        console.error('Failed to update insight:', data);
      }
      if (callback) callback(data, status);
    }
  );
}

export function wsDeleteInsight(id: number, callback?: Function) {

  Broker.instance().sendMessage(
    {
      key: 'insight',
      subKey: 'delete',
      body: { id },
    },
    (data: any, status: string) => {
      if (status === 'Bad Request') {
        console.error('Failed to delete insight:', data);
      }
      if (callback) callback(data, status);
    }
  );
}

export function wsInsightSubscription(callback: (data: any, status: string) => void) {
  const id = Broker.instance().subscribe(callback, 'insight_result');
  return () => {
    Broker.instance().unSubscribe('insight_result', id);
  };
}
